@@ -42,7 +42,9 @@
 #define WINDOW_PADDING 5
 #define TOOLTIP_PADDING 5
 
-#define BAOBAB_RINGSCHART_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), BAOBAB_RINGSCHART_TYPE, BaobabRingschartPrivate))
+#define BAOBAB_RINGSCHART_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), \
+                                            BAOBAB_RINGSCHART_TYPE, \
+                                            BaobabRingschartPrivate))
 
 G_DEFINE_TYPE (BaobabRingschart, baobab_ringschart, GTK_TYPE_WIDGET);
 
@@ -52,8 +54,6 @@ enum {
   RIGHT_BUTTON = 3
 };
 
-typedef struct _BaobabRingschartPrivate BaobabRingschartPrivate;
-
 struct _BaobabRingschartPrivate {
   GtkTreeModel *model;
   gdouble center_x;
@@ -100,11 +100,10 @@ struct _Tooltip {
 };
 
 /* Signals */
-enum
-  {
-    SECTOR_ACTIVATED,
-    LAST_SIGNAL
-  };
+enum {
+  SECTOR_ACTIVATED,
+  LAST_SIGNAL
+};
 
 static guint ringschart_signals [LAST_SIGNAL] = { 0 };
 
@@ -272,10 +271,13 @@ baobab_ringschart_class_init (BaobabRingschartClass *class)
 }
 
 static void
-baobab_ringschart_init (BaobabRingschart *object)
+baobab_ringschart_init (BaobabRingschart *rchart)
 {
-  BaobabRingschartPrivate *priv = BAOBAB_RINGSCHART_GET_PRIVATE(object);
- 
+  BaobabRingschartPrivate *priv;
+
+  priv = BAOBAB_RINGSCHART_GET_PRIVATE (rchart);
+  rchart->priv = priv;
+
   priv->model = NULL;
   priv->center_x = 0;  
   priv->center_y = 0;
@@ -305,25 +307,15 @@ baobab_ringschart_init (BaobabRingschart *object)
 static void
 baobab_ringschart_dispose (GObject *object)
 {
-  BaobabRingschartPrivate *priv = BAOBAB_RINGSCHART_GET_PRIVATE(object);
-  
+  BaobabRingschartPrivate *priv;
+
+  priv = BAOBAB_RINGSCHART (object)->priv;
+
   if (priv->model)
     {
-      g_signal_handlers_disconnect_by_func (priv->model,
-					    baobab_ringschart_row_changed,
-					    object);
-      g_signal_handlers_disconnect_by_func (priv->model,
-					    baobab_ringschart_row_inserted,
-					    object);
-      g_signal_handlers_disconnect_by_func (priv->model,
-					    baobab_ringschart_row_has_child_toggled,
-					    object);
-      g_signal_handlers_disconnect_by_func (priv->model,
-					    baobab_ringschart_row_deleted,
-					    object);
-      g_signal_handlers_disconnect_by_func (priv->model,
-					    baobab_ringschart_rows_reordered,
-					    object);
+      baobab_ringschart_disconnect_signals (GTK_WIDGET (object),
+					    priv->model);
+
       g_object_unref (priv->model);
 
       priv->model = NULL;
@@ -383,7 +375,7 @@ baobab_ringschart_size_allocate (GtkWidget     *widget,
   g_return_if_fail (BAOBAB_IS_RINGSCHART (widget));
   g_return_if_fail (allocation != NULL);
 
-  priv = BAOBAB_RINGSCHART_GET_PRIVATE (widget);
+  priv = BAOBAB_RINGSCHART (widget)->priv;
 
   widget->allocation = *allocation;
 
@@ -424,6 +416,7 @@ baobab_ringschart_set_property (GObject         *object,
       baobab_ringschart_set_root (GTK_WIDGET (rings_chart), g_value_get_object (value));
       break;
     default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
     }
 }
@@ -436,7 +429,7 @@ baobab_ringschart_get_property (GObject    *object,
 {
   BaobabRingschartPrivate *priv;
 
-  priv = BAOBAB_RINGSCHART_GET_PRIVATE (object);
+  priv = BAOBAB_RINGSCHART (object)->priv;
 
   switch (prop_id)
     {
@@ -466,7 +459,7 @@ baobab_ringschart_update_draw (BaobabRingschart* rchart,
   if (!GTK_WIDGET_REALIZED (rchart))
     return;
 
-  priv = BAOBAB_RINGSCHART_GET_PRIVATE (rchart);
+  priv = BAOBAB_RINGSCHART (rchart)->priv;
 
   if (priv->root != NULL)
     {
@@ -566,7 +559,8 @@ baobab_ringschart_expose (GtkWidget *rchart, GdkEventExpose *event)
 {
   cairo_t *cr;
   BaobabRingschartPrivate *priv;
-  priv = BAOBAB_RINGSCHART_GET_PRIVATE (rchart);
+
+  priv = BAOBAB_RINGSCHART (rchart)->priv;
 
   /* the columns are not set we paint nothing */
   if (priv->name_column == priv->percentage_column)
@@ -621,12 +615,12 @@ baobab_ringschart_draw (GtkWidget *rchart, cairo_t *cr)
   BaobabRingschartPrivate *priv;
   GtkTreeIter initial_iter = {0};  
   GSList *tooltips = NULL;
-  gfloat perc;
+  gdouble perc;
   gdouble max_angle;
 
   cairo_stroke (cr); 
 
-  priv = BAOBAB_RINGSCHART_GET_PRIVATE (rchart);
+  priv = BAOBAB_RINGSCHART (rchart)->priv;
 
   if (priv->root != NULL)
     {
@@ -658,7 +652,7 @@ baobab_ringschart_draw (GtkWidget *rchart, cairo_t *cr)
     {
       gtk_tree_model_get (priv->model, &initial_iter, 
                           priv->percentage_column, &perc, -1);
-      
+
       max_angle = 2*G_PI*perc*0.01;
     }
 
@@ -713,7 +707,6 @@ baobab_ringschart_draw (GtkWidget *rchart, cairo_t *cr)
     while (g_slist_next(tooltips));
 }
 
-
 static void 
 draw_sector (cairo_t *cr, 
              gdouble center_x, 
@@ -961,7 +954,7 @@ tree_traverse (cairo_t *cr,
   GtkTreeIter child;
   gboolean highlighted = FALSE;
 
-  priv = BAOBAB_RINGSCHART_GET_PRIVATE (rchart);
+  priv = BAOBAB_RINGSCHART (rchart)->priv;
 
   if (final_angle - init_angle < MIN_DRAWABLE_ANGLE)
     return;
@@ -1054,24 +1047,30 @@ tree_traverse (cairo_t *cr,
       gtk_tree_model_iter_children (priv->model, &child, iter);
 
       _init_angle = init_angle;
-      do 
+
+      do
         {
-          gfloat size;
-          guint next_radius = depth == 0 ? radius : radius+priv->thickness;
+          gdouble size;
+          guint next_radius;
 
-          gtk_tree_model_get (priv->model, &child, priv->percentage_column, &size, -1);
-	 
-          _final_angle = _init_angle + (final_angle - init_angle) * size/100;
+          next_radius = depth == 0 ? radius : radius+priv->thickness;
 
-          tree_traverse (cr, rchart, &child, depth+1, next_radius, 
-                         _init_angle, _final_angle, 
-                         tooltips, real_draw);
+          gtk_tree_model_get (priv->model, &child,
+                              priv->percentage_column, &size, -1);
 
-          _init_angle = _final_angle;
+          if (size >= 0)
+            {
+               _final_angle = _init_angle + (final_angle - init_angle) * size/100;
+
+               tree_traverse (cr, rchart, &child, depth+1, next_radius, 
+                              _init_angle, _final_angle, 
+                              tooltips, real_draw);
+
+               _init_angle = _final_angle;
+            }
         }
       while (gtk_tree_model_iter_next (priv->model, &child));
     }
-
 }
 
 static gint 
@@ -1081,7 +1080,7 @@ baobab_ringschart_button_release (GtkWidget *widget,
   BaobabRingschartPrivate *priv;
   GtkTreeIter parent_iter;
 
-  priv = BAOBAB_RINGSCHART_GET_PRIVATE (widget);
+  priv = BAOBAB_RINGSCHART (widget)->priv;
 
   switch (event->button)
     {      
@@ -1144,34 +1143,31 @@ baobab_ringschart_motion_notify (GtkWidget        *widget,
 {
   BaobabRingschartPrivate *priv;
   gdouble angle;
-  gdouble x = event->x;
-  gdouble y = event->y;
+  gdouble x, y;
   guint depth;
-  
-  priv = BAOBAB_RINGSCHART_GET_PRIVATE (widget);
 
-  x -= priv->center_x;
-  y -= priv->center_y;
+  priv = BAOBAB_RINGSCHART (widget)->priv;
+
+  x = event->x - priv->center_x;
+  y = event->y - priv->center_y;
 
   /* FIXME: could we use cairo_in_stroke or cairo_in_fill? would it be
      a better solution */
-  angle = -1.0 * atan2 (x, y) + G_PI_2;
+  angle = atan2 (y, x);
   angle = (angle > 0) ? angle : angle + 2*G_PI;
-  
+
   depth = sqrt (x*x + y*y) / (priv->thickness);
 
-  /* we use +1 because we want to control the movement to the layer
-     over the last one */
-  if (!(((priv->current_depth == depth) || (depth > priv->max_depth+1)) 
-        && (priv->current_init_angle < angle)                             
-        && (angle < priv->current_final_angle)))                          
+  if (priv->current_depth != depth     ||
+      angle < priv->current_init_angle ||
+      angle > priv->current_final_angle)
     {
       priv->current_depth = depth;
       priv->point_angle = angle;
-             
-      gtk_widget_queue_draw(widget);
+
+      gtk_widget_queue_draw (widget);
     }
-  else 
+  else
     {
       gdk_window_get_pointer (widget->window, NULL, NULL, NULL);
     }
@@ -1179,7 +1175,6 @@ baobab_ringschart_motion_notify (GtkWidget        *widget,
   return FALSE;
 }
 
-
 static inline void
 baobab_ringschart_disconnect_signals (GtkWidget *rchart, 
                                       GtkTreeModel *model)
@@ -1293,7 +1288,7 @@ baobab_ringschart_set_model_with_columns (GtkWidget *rchart,
   g_return_if_fail (BAOBAB_IS_RINGSCHART (rchart));
   g_return_if_fail (GTK_IS_TREE_MODEL (model));
 
-  priv = BAOBAB_RINGSCHART_GET_PRIVATE (rchart);
+  priv = BAOBAB_RINGSCHART (rchart)->priv;
 
   baobab_ringschart_set_model (rchart, model);
   
@@ -1335,8 +1330,8 @@ baobab_ringschart_set_model (GtkWidget *rchart,
   g_return_if_fail (BAOBAB_IS_RINGSCHART (rchart));
   g_return_if_fail (GTK_IS_TREE_MODEL (model));
 
-  priv = BAOBAB_RINGSCHART_GET_PRIVATE (rchart);
-  
+  priv = BAOBAB_RINGSCHART (rchart)->priv;
+
   if (model == priv->model)
     return;
 
@@ -1378,12 +1373,9 @@ baobab_ringschart_set_model (GtkWidget *rchart,
 GtkTreeModel *
 baobab_ringschart_get_model (GtkWidget *rchart)
 {
-  BaobabRingschartPrivate *priv;
-
   g_return_val_if_fail (BAOBAB_IS_RINGSCHART (rchart), NULL);
 
-  priv = BAOBAB_RINGSCHART_GET_PRIVATE (rchart);
-  return priv->model;
+  return BAOBAB_RINGSCHART (rchart)->priv->model;
 }
 
 /**
@@ -1405,8 +1397,8 @@ baobab_ringschart_set_max_depth (GtkWidget *rchart,
 
   g_return_if_fail (BAOBAB_IS_RINGSCHART (rchart));
 
-  priv = BAOBAB_RINGSCHART_GET_PRIVATE (rchart);
-  
+  priv = BAOBAB_RINGSCHART (rchart)->priv;
+
   if (max_depth != priv->max_depth)
     {    
       priv->max_depth = MIN (max_depth,
@@ -1430,12 +1422,9 @@ baobab_ringschart_set_max_depth (GtkWidget *rchart,
 guint
 baobab_ringschart_get_max_depth (GtkWidget *rchart)
 {
-  BaobabRingschartPrivate *priv;
-
   g_return_val_if_fail (BAOBAB_IS_RINGSCHART (rchart), 0);
 
-  priv = BAOBAB_RINGSCHART_GET_PRIVATE (rchart);
-  return priv->max_depth;
+  return BAOBAB_RINGSCHART (rchart)->priv->max_depth;
 }
 
 /**
@@ -1458,7 +1447,7 @@ baobab_ringschart_set_root (GtkWidget *rchart,
 
   g_return_if_fail (BAOBAB_IS_RINGSCHART (rchart));
 
-  priv = BAOBAB_RINGSCHART_GET_PRIVATE (rchart);
+  priv = BAOBAB_RINGSCHART (rchart)->priv;
 
   g_return_if_fail (priv->model != NULL);
 
@@ -1485,12 +1474,9 @@ baobab_ringschart_set_root (GtkWidget *rchart,
 GtkTreePath *
 baobab_ringschart_get_root (GtkWidget *rchart)
 {
-  BaobabRingschartPrivate *priv;
-
   g_return_val_if_fail (BAOBAB_IS_RINGSCHART (rchart), NULL);
 
-  priv = BAOBAB_RINGSCHART_GET_PRIVATE (rchart);
-  return gtk_tree_row_reference_get_path (priv->root);  
+  return gtk_tree_row_reference_get_path (BAOBAB_RINGSCHART (rchart)->priv->root);  
 }
 
 /**
@@ -1505,15 +1491,11 @@ baobab_ringschart_get_root (GtkWidget *rchart)
 gint 
 baobab_ringschart_get_drawn_elements (GtkWidget *rchart)
 {
-  BaobabRingschartPrivate *priv;
-
   g_return_val_if_fail (BAOBAB_IS_RINGSCHART (rchart), 0);
 
-  priv = BAOBAB_RINGSCHART_GET_PRIVATE (rchart);
-  return priv->drawn_elements;  
+  return BAOBAB_RINGSCHART (rchart)->priv->drawn_elements;  
 }
 
-
 /**
  * baobab_ringschart_freeze_updates:
  * @rchart: the #BaobabRingschart whose model signals are going to be frozen.
@@ -1533,7 +1515,7 @@ baobab_ringschart_freeze_updates (GtkWidget *rchart)
 
   g_return_if_fail (BAOBAB_IS_RINGSCHART (rchart));
 
-  priv = BAOBAB_RINGSCHART_GET_PRIVATE (rchart);
+  priv = BAOBAB_RINGSCHART (rchart)->priv;
 
   if (!priv->is_frozen) 
     {
@@ -1596,8 +1578,8 @@ baobab_ringschart_thaw_updates (GtkWidget *rchart)
 
   g_return_if_fail (BAOBAB_IS_RINGSCHART (rchart));
 
-  priv = BAOBAB_RINGSCHART_GET_PRIVATE (rchart);
-  
+  priv = BAOBAB_RINGSCHART (rchart)->priv;
+
   if (priv->is_frozen)
     {
       if (priv->model)
@@ -1632,8 +1614,8 @@ baobab_ringschart_set_init_depth (GtkWidget *rchart,
   BaobabRingschartPrivate *priv;
   
   g_return_if_fail (BAOBAB_IS_RINGSCHART (rchart));
-  
-  priv = BAOBAB_RINGSCHART_GET_PRIVATE (rchart);
+
+  priv = BAOBAB_RINGSCHART (rchart)->priv;
 
   priv->init_depth = depth;
   gtk_widget_queue_draw (rchart); 
@@ -1654,8 +1636,8 @@ void baobab_ringschart_draw_center (GtkWidget *rchart,
   BaobabRingschartPrivate *priv;
   
   g_return_if_fail (BAOBAB_IS_RINGSCHART (rchart));
-  
-  priv = BAOBAB_RINGSCHART_GET_PRIVATE (rchart);
+
+  priv = BAOBAB_RINGSCHART (rchart)->priv;
 
   priv->draw_center = draw_center;
   gtk_widget_queue_draw (rchart);   
